"""Antelopy

Python package to handle Antelope action data serialization

See https://github.com/stuckatsixpm/antelopy for examples."""
from antelopy.cache.abicache import AbiCache

__all__ = ["AbiCache"]
